/***************************************************************************
                          fleetview.cpp  -  description
                             -------------------
    begin                : Sun Mar 10 2002
    copyright            : (C) 2002 by Michael Andreen
    email                : whale@linux.nu
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "fleetview.h"

#include "ui/roidseditview.h"
#include "ui/resourceview.h"
#include "planet.h"

#include <qvalidator.h>

#include <iostream>
#include <map>

using namespace std;

FleetView::FleetView(const Fleet* fleet, bool friendly, QWidget *parent, const char *name ) : QWidget(parent,name)
{
	m_iMinTick = 0;
	m_iMaxTick = 0;
	m_iResultMode = 0;

	m_Modified = false;
	const Planet* planet = 0;
	if((planet = dynamic_cast<const Planet*>(fleet)))
	{
			m_Fleet = new Planet(*planet);
			m_bHome = true;
	}
	else
	{
		m_Fleet = new Fleet(*fleet);
		m_bHome = false;
	}
	
	m_Fleet->printFleet();
	m_bFriendly = friendly;

	//m_ChangesMapper = new QSignalMapper(this);

	
	findNames();
	int rows = m_Names.size() / 3 + m_Names.size() % 3;
	m_MainLayout = new QVBoxLayout(this,2);
	m_MainLayout->addSpacing(2);
	m_UnitsLayout = new QGridLayout(m_MainLayout, rows, 6);
	buildUnitTable();
	//QSpacerItem* space = new QSpacerItem(1, 1);
	

	m_MainLayout->addStretch(10);

	if (m_bHome)
	{
		m_RoidsEditView = new RoidsEditView(this);
		m_MainLayout->addWidget(m_RoidsEditView);
		connect(m_RoidsEditView, SIGNAL(changed(const QString&, int)), SLOT(slotRoidsChanged(const QString&, int)));
	}
	m_ResourceView = new ResourceView(this);
	m_MainLayout->addWidget(m_ResourceView);
  	fillTable();
}

FleetView::~FleetView(){
}

/////////////////////////////////////////////////////////////////////
//
void FleetView::viewFleet(const Fleet* fleet, bool friendly)
{
	if (m_Fleet->race() != fleet->race())
	{
		findNames();
	}
	delete m_Fleet;
	if(m_bHome)
	{
		m_Fleet = new Planet(*dynamic_cast<const Planet*>(fleet));
	}
	else
	{
		m_Fleet = new Fleet(*fleet);
	}
	//m_Fleet->printFleet();
	fillTable();
}

/////////////////////////////////////////////////////////////////////
//
void FleetView::findNames()
{
	m_Names.clear();

	const UnitList& units = Fleet::Units();	
	vector<int> races = m_Fleet->RacesAllowed();

	for (UnitList::const_iterator i = units.begin(); i != units.end(); ++i)
	{
		for (vector<int>::iterator j = races.begin(); j != races.end(); ++j)
		{
			if ((*j) == i->second.race())
			{
				m_Names.push_back(i->first.c_str());
				break;
			}
		}
	}
}

/////////////////////////////////////////////////////////////////////
//
void FleetView::buildUnitTable()
{
	int rows = 0;
	QIntValidator* val = new QIntValidator(this);
	val->setBottom(0);

	for (vector<QString>::iterator i = m_Names.begin(); i != m_Names.end(); ++rows)
	{
		for ( int cols = 0; i != m_Names.end() && cols < 3; ++i, ++cols)
		{
			m_UnitsLabel[(*i)] = new QLabel((*i), this);
			m_UnitsLayout->addWidget(m_UnitsLabel[(*i)], rows, cols*3);

			m_UnitsEdit[(*i)] = new QLineEdit(this,(*i));
      m_UnitsEdit[(*i)]->setValidator(val);
			m_UnitsLayout->addWidget(m_UnitsEdit[(*i)], rows, cols*3+1);

			m_UnitsLostSurvivedView[(*i)] = new QLineEdit(this);
			m_UnitsLostSurvivedView[(*i)]->setFocusPolicy(QWidget::NoFocus);
			m_UnitsLostSurvivedView[(*i)]->setReadOnly(true);
			m_UnitsLayout->addWidget(m_UnitsLostSurvivedView[(*i)], rows, cols*3+2);

			connect(m_UnitsEdit[(*i)], SIGNAL(textChanged(const QString&)),this, SLOT(unitsChanged(const QString&)));
		}
	}

}

/////////////////////////////////////////////////////////////////////
//
const Fleet* FleetView::fleet()
{
	return m_Fleet;
}

/////////////////////////////////////////////////////////////////////
//
bool FleetView::isHome()
{
	return m_bHome;
}

/////////////////////////////////////////////////////////////////////
//
bool FleetView::isFriendly()
{
	return m_bFriendly;
}

/////////////////////////////////////////////////////////////////////
//
void FleetView::unitsChanged(const QString& number)
{
	m_Fleet->setFleet(sender()->name(), number.toInt());
	m_Modified = true;
	emit fleetChanged(m_Fleet);
}

/////////////////////////////////////////////////////////////////////
//
bool FleetView::isModified()
{
	return m_Modified;
}

/////////////////////////////////////////////////////////////////////
//
void FleetView::fillTable()
{
	for(vector<QString>::iterator i = m_Names.begin(); i != m_Names.end(); ++i)
	{
		m_UnitsEdit[(*i)]->setText(QString("%1").arg(m_Fleet->fleet(i->latin1(), 0)));
	}
	Planet* pl;
	if ((pl = dynamic_cast<Planet*>(m_Fleet)) && m_bHome)
	{
		m_RoidsEditView->setValue(tr("score"),pl->planetScore());
		m_RoidsEditView->setValue(tr("metal"),pl->roids(tr("metal").latin1()));
		m_RoidsEditView->setValue(tr("crystal"),pl->roids(tr("crystal").latin1()));
		m_RoidsEditView->setValue(tr("eonium"),pl->roids(tr("eonium").latin1()));
		m_RoidsEditView->setValue(tr("uninit"),pl->roids(tr("uninit").latin1()));
	}
	slotViewTickRange();
}

/////////////////////////////////////////////////////////////////////
//
void FleetView::slotRoidsChanged(const QString& type, int value)
{
	Planet* pl;
	if ((pl = dynamic_cast<Planet*>(m_Fleet)))
	{
		if (type == tr("score"))
			pl->setPlanetScore(value);
		else
		{
			pl->setRoids(type.latin1(), value);
		}
		emit fleetChanged(pl);
	}
}

/////////////////////////////////////////////////////////////////////
//
void FleetView::slotViewTickRange(int min = -1, int max = -1)
{
	if (min > -1)
		m_iMinTick = min;
	if (max > -1)
		m_iMaxTick = max;

	int before = 0;
	int after  = 0;
	for(vector<QString>::iterator i = m_Names.begin(); i != m_Names.end(); ++i)
	{
		int show;
		if (m_iResultMode == 1) //Blocked
		{
			show = m_Fleet->blockedFleet(i->latin1(), m_iMaxTick);
		}
		else if (m_iResultMode == 2) //Survived
		{
			show = m_Fleet->fleet(i->latin1(), m_iMaxTick);
		}
		else
		{
			before = m_Fleet->fleet(i->latin1(), m_iMinTick);
			after = m_Fleet->fleet(i->latin1(), m_iMaxTick);
			show = after - before;
		}
		m_UnitsLostSurvivedView[(*i)]->setText(QString("%1").arg( show ));
	}

	int lost;
	
	int init = 0;

	before = m_Fleet->resource(tr("metal").latin1(),m_iMinTick);
	after = m_Fleet->resource(tr("metal").latin1(),m_iMaxTick);
	lost = after - before;
	init += lost;
	m_ResourceView->setLines(tr("metal"),lost);

	before = m_Fleet->resource(tr("crystal").latin1(),m_iMinTick);
	after = m_Fleet->resource(tr("crystal").latin1(),m_iMaxTick);
	lost = after - before;
	init += lost;
	m_ResourceView->setLines(tr("crystal"),lost);

	before = m_Fleet->resource(tr("eonium").latin1(),m_iMinTick);
	after = m_Fleet->resource(tr("eonium").latin1(),m_iMaxTick);
	lost = after - before;
	init += lost;
	m_ResourceView->setLines(tr("eonium"),lost);

	before = m_Fleet->resource(tr("uninit").latin1(),m_iMinTick);
	after = m_Fleet->resource(tr("uninit").latin1(),m_iMaxTick);
	lost = after - before;
	m_ResourceView->setLines(tr("uninit"),lost);

	before = m_Fleet->score(m_iMinTick);
	after = m_Fleet->score(m_iMaxTick);
	lost = after - before;
	m_ResourceView->setLines(tr("score"),after);
	m_ResourceView->setLines(tr("lost"),lost);
	float cost =  (before - after) / float(init) / 1000;
	m_ResourceView->setLines(tr("cost"),cost);
	


	Planet* pl;
	if ((pl = dynamic_cast<Planet*>(m_Fleet)))
	{

		before = pl->roids(tr("metal").latin1(),m_iMinTick);
		after = pl->roids(tr("metal").latin1(),m_iMaxTick);
		lost = after - before;
		m_RoidsEditView->slotSetLost(tr("metal"),lost);

		before = pl->roids(tr("crystal").latin1(),m_iMinTick);
		after = pl->roids(tr("crystal").latin1(),m_iMaxTick);
		lost = after - before;
		m_RoidsEditView->slotSetLost(tr("crystal"),lost);

		before = pl->roids(tr("eonium").latin1(),m_iMinTick);
		after = pl->roids(tr("eonium").latin1(),m_iMaxTick);
		lost = after - before;
		m_RoidsEditView->slotSetLost(tr("eonium"),lost);

		before = pl->roids(tr("uninit").latin1(),m_iMinTick);
		after = pl->roids(tr("uninit").latin1(),m_iMaxTick);
		lost = after - before;
		m_RoidsEditView->slotSetLost(tr("uninit"),lost);
	}
}

//////////////////////////////////////////////////////////////////////////
//
void FleetView::slotSetResultMode(int i)
{
	m_iResultMode = i;
	slotViewTickRange();
}
