/***************************************************************************
                          planet.cpp  -  description
                             -------------------
    begin                : Wed Apr 3 2002
    copyright            : (C) 2002 by Michael Andreen
    email                : whale@linux.nu
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "planet.h"

#include <iostream>
using namespace std;

Planet::Planet()
{
	m_sRace = "Planet";
}

Planet::~Planet(){
}

//////////////////////////////////////////////////////////////////////////
//
unsigned Planet::planetScore(int tick = 0) const
{
	int ticks = m_Score.size();
	if (ticks == 0)
		return 0;

	--ticks;

	if (ticks < tick)
		return m_Score[ticks];

	return m_Score[tick];
}

//////////////////////////////////////////////////////////////////////////
//
void Planet::setPlanetScore(unsigned number, int tick = 0)
{
	int earlier = 0;
	int ticks = m_Score.size();

	if (ticks != 0)
		earlier = m_Score[ticks - 1];

	for (int i = ticks; i <= tick; ++i)
	{
		m_Score.push_back(earlier);
	}
	m_Score[tick] = number;
}

//////////////////////////////////////////////////////////////////////////
//
void Planet::addPlanetScore(unsigned number, int tick = 0)
{
	int earlier = 0;
	int ticks = m_Score.size();

	if (ticks != 0)
		earlier = m_Score[ticks - 1];

	for (int i = ticks; i <= tick; ++i)
	{
		m_Score.push_back(earlier);
	}
	m_Score[tick] += number;
}
//////////////////////////////////////////////////////////////////////////
//
int Planet::roids(std::string type, int tick = 0) const
{
	// const... I would like [] as for const types: int ticks = m_Roids[type].size();
	
	vector<int>const* roids = 0;
	for (RoidList::const_iterator i = m_Roids.begin(); i != m_Roids.end(); ++i)
	{
		if (i->first == type)
		{
			roids = &i->second;
			break;
		}
	}
	if (roids == 0)
		return 0;

	int ticks = roids->size();

	if( ticks == 0)
		return 0;
	--ticks;
	if (ticks < tick)
		return roids->at(ticks);
	return roids->at(tick);
}

//////////////////////////////////////////////////////////////////////////
//
void Planet::setRoids(std::string type, int number, int tick = 0)
{
	int ticks = m_Roids[type].size();
	int roids = 0;
	if (m_Roids[type].size() > 0)
		roids = m_Roids[type][m_Roids[type].size() - 1];

	for (int i = ticks; i <= tick; ++i )
		m_Roids[type].push_back(roids);
	m_Roids[type][tick] = number;
}

//////////////////////////////////////////////////////////////////////////
//
void Planet::takeRoids(std::string type, int number, int tick = 0)
{
	int ticks = m_Roids[type].size();
	int roids = 0;
	if (m_Roids[type].size() > 0)
		roids = m_Roids[type][m_Roids[type].size() - 1];

	for (int i = ticks; i <= tick; ++i )
		m_Roids[type].push_back(roids);
	m_Roids[type][tick] -= number;
}



//////////////////////////////////////////////////////////////////////////
//
void Planet::runBattle(std::vector<Fleet*> friendly, std::vector<Fleet*> hostile)
{
	if (hostile.size() == 0)
			return;

	int skipped = 0;
	for(int tick = 1; skipped < 20; ++tick)
	{
		//See who's in the battle at the current tick
		vector<Fleet*> friends = calculateSide(friendly, 6, tick);
		vector<Fleet*> hostiles = calculateSide(hostile, 3, tick);

		// No idea to calculate anything if noone is there.. ;)
		if (hostiles.size() == 0)
		{
			skipped++;
			continue;
		}
		else
			skipped = 0;

		Planet allFriends;
		allFriends.addToThis(friends, tick - 1);

		/*for (RoidList::iterator i = m_Roids.begin(); i != m_Roids.end(); ++i)
			allFriends.setRoids(i->first, roids(i->first, tick));

		allFriends.setPlanetScore(m_iScore);*/
		
		Fleet allHostiles;
		allHostiles.addToThis(hostiles, tick - 1);

		map<string, map<string, int> > stealfriendly;
		map<string, map<string, int> > stealhostile;
		
		calcOneTick(&allFriends, &allHostiles, stealfriendly, stealhostile, tick );

		//allFriends.printFleet();

		allFriends.distributeLossesGains(friends, tick);
		allHostiles.distributeLossesGains(hostiles, tick);
		allHostiles.distributeCappedRoids(hostiles, tick);

		calculateScoreLoss(friends, tick);

	/*	for (RoidList::iterator i = m_Roids.begin(); i != m_Roids.end(); ++i)
			setRoids(i->first, roids(i->first, 1), tick);*/
	}

	for (RoidList::iterator i = m_Roids.begin(); i != m_Roids.end(); ++i)
	{
		for (vector<int>::iterator j = i->second.begin(); j != i->second.end(); ++j)
			cout << i->first << " roids : " << (*j) << endl;
	}
}

//////////////////////////////////////////////////////////////////////////
//
void Planet::calcOneTick(Planet* friendly, Fleet* hostile, std::map<std::string, std::map<std::string, int> >& stealfriendly, std::map<std::string, std::map<std::string, int> >&  stealhostile, int tick = 0)
{
	map<int, string> unitsinit; // order units after their ininitiative
	for (UnitList::iterator i = s_Units.begin(); i != s_Units.end(); ++i)
		unitsinit[i->second.initiative()] = i->first;

	for (map<int, string>::iterator i = unitsinit.begin(); i != unitsinit.end(); ++i)
	{
		Fleet* hostiletemp = new Fleet(*hostile);
		Planet* friendlytemp = new Planet(*friendly);
		
		string unittype = i->second;

		//cerr << "Initiative: " << s_Units[unittype].initiative() << " with unit: " << unittype << endl;

		if (s_Units[unittype].type() == "EMP")	
		{
			hostiletemp->takeEMP(unittype, friendly->freeFleet(unittype, 1));
			friendlytemp->takeEMP(unittype, hostile->freeFleet(unittype, 1));
		}
		else if (s_Units[unittype].type() == "Steal")
		{
			hostiletemp->takeShoot(unittype, friendly->freeFleet(unittype, 1), stealfriendly[unittype]);
			friendlytemp->takeShoot(unittype, hostile->freeFleet(unittype, 1), stealhostile[unittype]);
		}
		else
		{
			map<string, int> temp;
			hostiletemp->takeShoot(unittype, friendly->freeFleet(unittype, 1), temp);
			friendlytemp->takeShoot(unittype, hostile->freeFleet(unittype, 1), temp);
		}

		if (s_Units[unittype].type() == "Pod")
		{
			if (planetScore(tick - 1) > 0)
			{
				float capping = float(planetScore(tick - 1)) / hostile->score() /  10;
			
				cerr << "Capping is: " << capping << endl;
	
				if (capping > 0.15)
					capping = 0.15;
				if (capping > 0)
				{
					for (RoidList::iterator roid = m_Roids.begin(); roid != m_Roids.end(); ++roid)
					{
						int caproids = capping * roids(roid->first, tick - 1);
						int freepods = hostiletemp->freeFleet(unittype, 1);
		
						cerr << "Possible to steal " << caproids << " " << roid->first << " roids\n";
						cerr << freepods << " free pods available\n";
						
						if (freepods <= 0)
							break;
						if (freepods < caproids)
							caproids = freepods;
		
						cerr << caproids << " roids stolen\n";
							
						takeRoids(roid->first, caproids, tick);
		
						//FIXME: Going to move this to the distribute roids section instead.. Not really move, I'll keep this, but "regenerate" the pods in the distribute ships function.
						hostiletemp->killFleet(unittype, caproids, 1);
						//int totroids = caproids + hostiletemp->resource(roid->first, 0);
						hostiletemp->addResource(roid->first, caproids, 1);
						
						cerr << caproids << " stolen " << roid->first << " roids\n";
					}
				}
			}
		}

		//set the the objects so they point at the modified objects
		*friendly = *friendlytemp;
		delete friendlytemp;
		*hostile = *hostiletemp;
		delete hostiletemp;
	}
}

void Planet::calculateScoreLoss(std::vector<Fleet*> friendly, int tick = 1)
{
	for (vector<Fleet*>::iterator i = friendly.begin(); i != friendly.end(); ++i)
	{
		if ((*i)->name().find("Home") != string::npos)
		{
			int scorechange = (*i)->score(tick) - (*i)->score(tick - 1);
			addPlanetScore(scorechange, tick);
		}
	}
}

